"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
function getVisitorsPrepareTw({ types: t }, options) {
    return {
        visitor: {
            Program: {
                enter(path, state) {
                    const twqueue = [];
                    // We need this transforms to run before anything else
                    // So we traverse here instead of a in a visitor
                    path.traverse({
                        ImportDeclaration: (p) => {
                            visitorImportDeclaration({ types: t }, p, state);
                        },
                        TaggedTemplateExpression: (path) => {
                            const { tag } = path.node;
                            if (
                            //  hasImport(t, path.scope, state.file.opts.filename, 'tw', options) &&
                            t.isIdentifier(tag) &&
                                tag.name === 'tw') {
                                twqueue.push(path);
                            }
                        },
                        JSXAttribute: (path) => {
                            const { node } = path;
                            if (t.isJSXAttribute(node) &&
                                t.isJSXIdentifier(node.name, { name: 'tw' })) {
                                /* Convert to a tw="css``" */
                                if (t.isJSXOpeningElement(path.parent)) {
                                    twqueue.push(path);
                                }
                            }
                        }
                    });
                    twqueue.forEach((item) => {
                        if (t.isTaggedTemplateExpression(item.node)) {
                            item.replaceWith(util_1.wrapExpression({ types: t }, item.node, state.cssImportLocalName));
                        }
                        else {
                            processJsxAttribute({ types: t }, item, state);
                        }
                    });
                    if (twqueue.length > 0) {
                        path.scope.crawl();
                    }
                },
                exit(nodePath, state) {
                    /** noop */
                }
            }
        }
    };
}
exports.default = getVisitorsPrepareTw;
function visitorImportDeclaration({ types: t }, path, state) {
    if (!t.isLiteral(path.node.source, { value: 'twstyled' })) {
        return;
    }
    let twImported;
    path.node.specifiers.forEach((specifier) => {
        // could be import or default import
        if (!t.isImportSpecifier(specifier)) {
            return;
        }
        const importedName = specifier.imported.name;
        if (importedName === 'tw') {
            state.twImportLocalName = specifier.local.name;
            twImported = specifier;
        }
        if (importedName === 'css') {
            state.cssImportLocalName = specifier.local.name;
        }
    });
    if (twImported && !state.cssImportLocalName) {
        path.node.specifiers.push(t.importSpecifier(t.identifier('css'), t.identifier('css')));
        state.cssImportLocalName = 'css';
    }
}
function processJsxAttribute({ types: t }, item, state) {
    const cxClassNameExpression = util_1.wrapExpression({ types: t }, item.node.value);
    // Check if a className props already exists
    const classNameAttribute = item.parent.attributes.find((a) => t.isJSXAttribute(a) && t.isJSXIdentifier(a.name, { name: 'className' }));
    const rightExpression = (classNameAttribute === null || classNameAttribute === void 0 ? void 0 : classNameAttribute.value) ||
        t.stringLiteral('');
    const result = util_1.combineExpressions({ types: t }, cxClassNameExpression, rightExpression);
    item.remove();
    if (t.isJSXAttribute(classNameAttribute)) {
        classNameAttribute.value = result;
    }
    else {
        /* Handle the case where no className exists yet */
        ;
        item.parent.attributes.unshift(t.jsxAttribute(t.jsxIdentifier('className'), t.isStringLiteral(result) ? result : result));
    }
}
//# sourceMappingURL=visitors-prepare-tailwind.js.map